#! /usr/bin/env python3

# coding: utf-8

import lzma, sys


def column_feature(column, line):
    return line[column]


def head_pos_feature(line, sentence):
    headid = int(line[6])
    if headid == 0:
        return "<root>"
    return sentence[headid - 1][4]


def feat_feature(feat, line):
    if len(line[5]) < 3:
        raise KeyError
    for elem in line[5].split("|"):
        try:
            k,v = elem.split("=")
            if k == feat:
                return v
        except ValueError: # misformed feature, happens e.g. in French
            continue
    raise KeyError


feature_col_map = {}
# done
feat_col_map_basque = {'case': 'KAS', 'gender': 'XXXNONEXXX', 'number': 'NUM', 'tense' : 'ASP'}
feat_col_map_french = {'case': 'XXXXNONEXXXX', 'gender': 'g', 'number': 'n', 'tense' : 't'}
feat_col_map_german = {'case': 'case', 'gender': 'gender', 'number': 'number', 'tense' : 'tense'}
feat_col_map_hungarian = {'case': 'Cas', 'gender': 'XXXNONEXXX', 'number': 'Num', 'tense' : 'Tense'}
feat_col_map_swedish = {'case': 'case', 'gender': 'gender', 'number': 'number', 'tense' : 'verbform'}


feat_col_map_polish = {'case': 'case', 'gender': 'gender', 'number': 'number', 'tense' : 'tense'}

# polish morph described here: http://www.ipipan.waw.pl/~wolinski/publ/eacl03-ws12.pdf
#There are 4 features that are not described in the 2003 paper: 'com' 'n' 'npun' 'pun', idk what they are
polish_morphs = {'case': ('nom','acc','gen','dat','inst','loc','voc'),
                 'gender': ('m1', 'm2', 'm3', 'f', 'n1', 'n2', 'p1', 'p2', 'p3'),
                 'number': ('sg', 'pl'),
                 'tense': ('imperf', 'perf')}



def polish_morph_feature(feat, line):
    morphlist = polish_morphs[feat]
    for i in line[5].split("|"):
        if i in morphlist:
            return i
    raise KeyError




feat_meta_map = {
"Basque" : feat_col_map_basque,
"French" : feat_col_map_french,
"German" : feat_col_map_german,
"Hungarian" : feat_col_map_hungarian,
"Polish" : feat_col_map_polish,
"Swedish" : feat_col_map_swedish
}




feature_map = {'pos': lambda line, s: column_feature(3, line),
               'label': lambda line, s: column_feature(7, line),
               'headpos': head_pos_feature,
               'case': lambda line, s: feat_feature(feature_col_map['case'], line),
               'gender': lambda line, s: feat_feature(feature_col_map['gender'], line),
               'number': lambda line, s: feat_feature(feature_col_map['number'], line),
               'tense': lambda line, s: feat_feature(feature_col_map['tense'], line),
}



def parse_arguments():
    import argparse
    parser = argparse.ArgumentParser(
        description='aligns word embeddings to a feature from conll data')
    parser.add_argument('-c', '--conll', required=True)
    parser.add_argument('-w', '--wordembeddings', required=True)
    parser.add_argument('-m', '--featuremap', required=True, help="Mapping between features (Strings) and IDs")
    parser.add_argument('-i', '--init_map', action='store_true', help="create featuremap instead of using an existing one")
    parser.add_argument('-f', '--feature', required=True,
                        choices=feature_map.keys())
    return parser.parse_args()


def read_embeddings(emb_fname):
    emb_file = lzma.open(emb_fname, 'rt') \
        if emb_fname.endswith("xz") else open(emb_fname)
    result = {}
    # ignore first line
    emb_file.readline()
    for line in emb_file:
        (k, v) = line.strip().split(' ', 1)
        v = " ".join(["%d:%s" % x for x in enumerate(v.split(), 1)])
        result[k] = v
    return result


def main():
    args = parse_arguments()

    global feature_col_map
    for (k,v) in feat_meta_map.items():
        if k in args.wordembeddings:
            feature_col_map = v
            break

    ffunc = feature_map[args.feature]
    # hack for Polish
    if "Polish" in args.wordembeddings and not "pos" in args.feature and not "label" == args.feature:
        ffunc = lambda line, s: polish_morph_feature(args.feature, line)

    sys.stderr.write("Reading embeddings...\n")
    embeddings = read_embeddings(args.wordembeddings)
    sys.stderr.write("...done\n")
    conllfile = open(args.conll)
    f_to_index = []
    if not args.init_map:
        with open(args.featuremap) as fmap:
            f_to_index = list([x.strip() for x in
                               fmap.readlines()])
    features = ""
    label = ""
    labelid = ""
    sentence = []
    num_lines = 0
    for line in conllfile:
        if (num_lines % 10000 == 0):
            sys.stderr.write("processed %d lines\n" %(num_lines,))
        num_lines += 1

        line = line.strip()
        if line != "":
            sentence.append(line.split("\t"))
            continue
        # if sentence is complete, process it
        for splitted in sentence:
            try:
                features = embeddings[splitted[1]]
                label = ffunc(splitted, sentence)
            except KeyError:
                continue  # ignore words withoud embedding or that feature
            try:
                labelid = str(f_to_index.index(label))
            except ValueError:
                if args.init_map:
                    labelid = str(len(f_to_index))
                    f_to_index.append(label)
                else:
                    # store nonexisting label
                    sys.stderr.write("label %s not seen during training!\n" % (label,))
                    labelid = "99999"
            sys.stdout.write(labelid + " ")
            sys.stdout.write(features + "\n")
        # empty sentence
        sentence = []
    #  end for conllfile
    mapfile = open(args.featuremap, "w")
    mapfile.writelines([x+"\n" for x in f_to_index])


if __name__ == "__main__":
    main()
