#! /bin/bash
# -*- tab-width: 4 -*-

set -e

datapath=/mnt/experiment/2015-word-emb
mkdir -p ${datapath}/cleaned
mkdir -p ${datapath}/embeddings
mkdir -p ${datapath}/mappings
mkdir -p ${datapath}/train
mkdir -p ${datapath}/test
mkdir -p ${datapath}/models
mkdir -p ${datapath}/results

# ---------------------- #
# Create word embeddings #
# ---------------------- #

# whether to run word embedding creation
RUNWTV=true

if [ truee = ${RUNWTV}e ]; then
	for language in Basque French English German Hungarian Polish Swedish; do
		if [ ! -e ${datapath}/cleaned/${language}.txt ]; then
			echo running ./cleanup.py ~/data/unlabeled/Unlabeled.${language}.pred.conll ${datapath}/cleaned/${language}.txt
			./cleanup.py ~/data/unlabeled/Unlabeled.${language}.pred.conll ${datapath}/cleaned/${language}.txt
		fi
		for window in 5 11; do
			for size in 10 100 200; do
				echo running on ${language} ${size} ${window}

				if [ ! -e ${datapath}/embeddings/${language}-cca-${size}-${window}.txt.xz ]; then
					tmpdir=${datapath}/cca-temp-$language
					mkdir -p $tmpdir
					echo running cca
					embfile=${datapath}/embeddings/${language}-cca-${size}-${window}.txt
					mkdir -p  ${datapath}/cca-temp/
					singular/singular --corpus ${datapath}/cleaned/${language}.txt --sentences --window $window --dim $size --rare 5 --output $tmpdir
					sleep 1
					mv $tmpdir/wordvectors_* $embfile
				 	sed 's/^[0-9]* //' $embfile | xz > ${embfile}.xz &
				fi


				# if [ ! -e ${datapath}/embeddings/${language}-glove-${size}-${window}.txt.xz ]; then
				# 	echo running glove
				#  	./run_glove.sh $datapath $language $size $window
				#  	sleep 1
				#  	xz ${datapath}/embeddings/${language}-glove-${size}-${window}.txt &
				# fi
				
				if [ ! -e ${datapath}/embeddings/${language}-brown-10-5.txt.xz ]; then
					echo running brown embedding
				 	./run_brown.sh $datapath $language 10
				 	sleep 1
				 	xz ${datapath}/embeddings/${language}-brown-10-5.txt
				fi

				if [ ! -e ${datapath}/embeddings/${language}-w2v-cbow-${size}-${window}.txt.xz ]; then
					echo running w2v-cbow
					word2vec/word2vec -train ${datapath}/cleaned/${language}.txt -output  ${datapath}/embeddings/${language}-w2v-cbow-${size}-${window}.txt -cbow 1 -size ${size} -threads 24 -debug 1
					sleep 1
					xz ${datapath}/embeddings/${language}-w2v-cbow-${size}-${window}.txt &
				fi

				if [ ! -e ${datapath}/embeddings/${language}-w2v-skip-${size}-${window}.txt.xz ]; then
					echo running w2v-skip
					word2vec/word2vec -train ${datapath}/cleaned/${language}.txt -output  ${datapath}/embeddings/${language}-w2v-skip-${size}-${window}.txt -cbow 0 -size ${size} -threads 24 -debug 1
					sleep 1
					xz ${datapath}/embeddings/${language}-w2v-skip-${size}-${window}.txt &
				fi

				
				if [ ! -e ${datapath}/embeddings/${language}-w2vf-${size}-5.txt.xz ]; then
					echo running w2v-dep
					if [ French = ${language} ]; then # French has no dependency information
						continue
					fi
					word2vecf/run_w2vf.sh $datapath $language $size 5
					sleep 1
					xz ${datapath}/embeddings/${language}-w2vf-${size}-5.txt &
				 fi
			done
		done
	done
fi





if [ -e run_train.sh ]; then
    rm run_train.sh
fi



for language in Basque English French German Hungarian Polish Swedish; do
	for emb in cca w2v-cbow w2v-skip glove w2vf brown; do
		for ftype in pos label headpos case gender tense number; do
			for window in 5 11; do
				for size in 10 100 200; do
					
					# skip features for which no annotation exists
					if [ Basque = $language -a $ftype = gender ]; then
						# Basque has no gender
						continue
					fi
					if [ French = $language -a $ftype = case ]; then
						continue
					fi
					if [ Hungarian = $language -a $ftype = gender ]; then
						continue
					fi
					if [ English = $language -a ! '(' $ftype = pos -o $ftype = label -o $ftype = headpos ')' ]; then
						# English has no morphological annotation
						continue
					fi

					if [ w2vf = $emb -a 11 = $window ]; then
						# w2vf has no window setting, 5 is inserted for all experiments
						continue
					fi

					if [ brown = $emb -a ! $window = 5 ]; then
						# brown has no window parameter
						continue
					fi
					if [ brown = $emb -a ! $size = 10 ]; then
						continue # upper limit for brown cluster
					fi
					if [ French = ${language} -a w2vf = $emb ]; then # French has no dependency information
						continue
					fi
					output=${datapath}/results/${language}-${ftype}-${emb}-${size}-${window}.results.txt
					if [ -e $output ]; then
						continue
					fi
					echo create_data and liblinear on ${language} ${emb} ${ftype} $window
					traindata=${datapath}/train/${language}-${ftype}-${emb}-${size}-${window}.train.txt
					testdata=${datapath}/test/${language}-${ftype}-${emb}-${size}-${window}.test.txt
					liblinearmodel=${datapath}/models/${language}-${ftype}-${emb}-${size}-${window}.model
					embedding=${datapath}/embeddings/${language}-${emb}-${size}-${window}.txt.xz


					if [ ! -e $embedding ]; then
						# try non-compressed file
						embedding=${datapath}/embeddings/${language}-${emb}-${size}-${window}.txt
					fi

					if [ ! -e $embedding ]; then
						echo skipping not existing $embedding
						continue
					fi

					# create_data is I/O bound, no benefit in parallelizing it
					if [ ! -e $traindata ]; then
						./create_data.py -c ~/data/spmrl/train.${language}.gold.conll -w $embedding  -f ${ftype} -m $datapath/mappings/${language}-${ftype}-${emb}.txt -i > ${traindata}
					fi
					if [ ! -e $testdata ]; then
						./create_data.py -c ~/data/spmrl/dev.${language}.gold.conll -w $embedding  -f ${ftype} -m $datapath/mappings/${language}-${ftype}-${emb}.txt > ${testdata}
					fi
					runprog="if [ ! -e $liblinearmodel ]; then echo train $liblinearmodel; liblinear-train -s 2 $traindata $liblinearmodel 2>&1 | tee ${traindata}.log; fi; echo test $liblinearmodel; liblinear-predict $testdata $liblinearmodel /dev/null > $output 2> ${testdata}.log"
					echo $runprog >> run_train.sh
					# echo liblinear-train -s 2 $traindata $liblinearmodel';' liblinear-predict $testdata $liblinearmodel /dev/null '>' $output >> run_train.sh
				done
			done
		done
	done
done
